@@ -126,9 +126,49 @@
 //!     }));
 //! }
 //! ```
+//!
+//! ### Path Semantical Quality
+//!
+//! Pocket-Prover has a model of [Path Semantical Quality](https://github.com/advancedresearch/path_semantics/blob/master/papers-wip2/path-semantical-quality.pdf)
+//! that resembles quantum logic.
+//!
+//! To write `x ~~ y` you use `q(x, y)` or `qual(x, y)`.
+//!
+//! `q(x, x)` is the same as `qubit(x)`.
+//! `q(x, y)` where `x` and `y` are symbolic distinct is the same as `eq(x, y)`.
+//!
+//! A qubit is a kind of "superposition".
+//! One can also think about it as introducing a new argument `qubit(x)` that depends on `x`.
+//!
+//! Since qubits can collide with other propositions,
+//! one must repeat measurements e.g. using `measure` to get classical states.
+//! However, sometimes one might wish to amplify quantum states, using `amplify` or `amp`.
+//!
+//! To use quality with path semantics, one should use `ps_core`.
+//! Path Semantical Logic is designed for equality, not quality.
+//!
+//! ```rust
+//! use pocket_prover::*;
+//!
+//! fn main() {
+//!     println!("Path semantics: {}", measure(1, || prove!(&mut |a, b, c, d| {
+//!         imply(
+//!             and!(
+//!                 ps_core(a, b, c, d),
+//!                 imply(a, c),
+//!                 imply(b, d)
+//!             ),
+//!             imply(qual(a, b), qual(c, d))
+//!         )
+//!     })));
+//! }
+//! ```
 
 pub mod extract;
 
+pub use qual as q;
+pub use amplify as amp;
+
 /// An AND relation of variable arguments.
 #[macro_export]
 macro_rules! and(
@@ -533,68 +573,138 @@ pub const P5: u64 = 0b11111111_11111111_11111111_11111111_00000000_00000000_0000
 /// Used to alternate higher than 6 arguments, set to `1`.
 pub const T: u64 = 0b11111111_11111111_11111111_11111111_11111111_11111111_11111111_11111111;
 
+/// Implemented by observables.
+pub trait Observable {
+    /// Gets the maximum energy level of observable.
+    fn max_energy() -> Self;
+    /// Gets the minimum energy level of two observables.
+    fn min_energy(self, other: Self) -> Self;
+}
+
+impl Observable for bool {
+    fn max_energy() -> bool {true}
+    fn min_energy(self, other: bool) -> bool {self && other}
+}
+
+impl Observable for u64 {
+    fn max_energy() -> u64 {std::u64::MAX}
+    fn min_energy(self, other: u64) -> u64 {self.min(other)}
+}
+
+/// Prepares a qubit using a proposition as seed.
+pub fn qubit(a: u64) -> u64 {
+    use rand::{Rng, SeedableRng};
+    use rand::rngs::StdRng;
+    let r = unsafe {&*current::Current::<u64>::new()};
+    let mut rng = StdRng::seed_from_u64(a ^ *r);
+    rng.gen()
+}
+
+/// Amplify a "wavefunction" of a proposition using its qubit transform.
+pub fn amplify(n: u32, mut a: u64) -> u64 {
+    for _ in 0..n {
+        a |= qubit(a);
+    }
+    a
+}
+
+/// Path semantical quality `a ~~ b`.
+pub fn qual(a: u64, b: u64) -> u64 {
+    and(eq(a, b), if a == b {qubit(a)} else {T})
+}
+
+/// Assumes the path semantical core axiom.
+pub fn ps_core(a: u64, b: u64, c: u64, d: u64) -> u64 {
+    imply(and!(qual(a, b), imply(a, c), imply(b, d)), qual(c, d))
+}
+
+/// Measures result repeatedly.
+pub fn measure<O: Observable>(n: u32, mut fun: impl FnMut() -> O) -> O {
+    let mut b = O::max_energy();
+    for _ in 0..n {
+        b = b.min_energy(fun());
+    }
+    b
+}
+
+fn call(mut fun: impl FnMut() -> u64) -> u64 {
+    let mut r = rand::random::<u64>();
+    let guard = current::CurrentGuard::new(&mut r);
+    let res = fun();
+    drop(guard);
+    res
+}
+
 /// Counts the number of solutions of a 1-argument boolean function.
-pub fn count1<F: FnMut(u64) -> u64>(f: &mut F) -> u32 {((f)(P0) & 0x3).count_ones()}
+pub fn count1<F: FnMut(u64) -> u64>(f: &mut F) -> u64 {
+    call(|| ((f)(P0) & 0x3)).count_ones() as u64
+}
 /// Counts the number of solutions of a 2-argument boolean function.
-pub fn count2<F: FnMut(u64, u64) -> u64>(f: &mut F) -> u32 {((f)(P0, P1) & 0xf).count_ones()}
+pub fn count2<F: FnMut(u64, u64) -> u64>(f: &mut F) -> u64 {
+    call(|| ((f)(P0, P1) & 0xf)).count_ones() as u64
+}
 /// Counts the number of solutions of a 3-argument boolean function.
-pub fn count3<F: FnMut(u64, u64, u64) -> u64>(f: &mut F) -> u32 {((f)(P0, P1, P2) & 0xff).count_ones()}
+pub fn count3<F: FnMut(u64, u64, u64) -> u64>(f: &mut F) -> u64 {
+    call(|| ((f)(P0, P1, P2) & 0xff)).count_ones() as u64
+}
 /// Counts the number of solutions of a 4-argument boolean function.
-pub fn count4<F: FnMut(u64, u64, u64, u64) -> u64>(f: &mut F) -> u32 {((f)(P0, P1, P2, P3) & 0xffff).count_ones()}
+pub fn count4<F: FnMut(u64, u64, u64, u64) -> u64>(f: &mut F) -> u64 {
+    call(|| ((f)(P0, P1, P2, P3) & 0xffff)).count_ones() as u64
+}
 /// Counts the number of solutions of a 5-argument boolean function.
-pub fn count5<F: FnMut(u64, u64, u64, u64, u64) -> u64>(f: &mut F) -> u32 {
-    ((f)(P0, P1, P2, P3, P4) & 0xffff_ffff).count_ones()
+pub fn count5<F: FnMut(u64, u64, u64, u64, u64) -> u64>(f: &mut F) -> u64 {
+    call(|| ((f)(P0, P1, P2, P3, P4) & 0xffff_ffff)).count_ones() as u64
 }
 /// Counts the number of solutions of a 6-argument boolean function.
-pub fn count6<F: FnMut(u64, u64, u64, u64, u64, u64) -> u64>(f: &mut F) -> u32 {
-    (f)(P0, P1, P2, P3, P4, P5).count_ones()
+pub fn count6<F: FnMut(u64, u64, u64, u64, u64, u64) -> u64>(f: &mut F) -> u64 {
+    call(|| (f)(P0, P1, P2, P3, P4, P5)).count_ones() as u64
 }
 /// Counts the number of solutions of a 7-argument boolean function.
-pub fn count7<F: FnMut(u64, u64, u64, u64, u64, u64, u64) -> u64>(f: &mut F) -> u32 {
-    (f)(P0, P1, P2, P3, P4, P5, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T).count_ones()
+pub fn count7<F: FnMut(u64, u64, u64, u64, u64, u64, u64) -> u64>(f: &mut F) -> u64 {
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T)).count_ones() as u64
 }
 /// Counts the number of solutions of an 8-argument boolean function.
-pub fn count8<F: FnMut(u64, u64, u64, u64, u64, u64, u64, u64) -> u64>(f: &mut F) -> u32 {
-    (f)(P0, P1, P2, P3, P4, P5, F, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, F, T).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, T).count_ones()
+pub fn count8<F: FnMut(u64, u64, u64, u64, u64, u64, u64, u64) -> u64>(f: &mut F) -> u64 {
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, T)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, T)).count_ones() as u64
 }
 /// Counts the number of solutions of a 9-argument boolean function.
-pub fn count9<F: FnMut(u64, u64, u64, u64, u64, u64, u64, u64, u64) -> u64>(f: &mut F) -> u32 {
-    (f)(P0, P1, P2, P3, P4, P5, F, F, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, F, F, T).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, F, T, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, F, T, T).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, F, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, F, T).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, T, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, T, T).count_ones()
+pub fn count9<F: FnMut(u64, u64, u64, u64, u64, u64, u64, u64, u64) -> u64>(f: &mut F) -> u64 {
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, F, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, F, T)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, T, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, T, T)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, F, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, F, T)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, T, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, T, T)).count_ones() as u64
 }
 /// Counts the number of solutions of a 10-argument boolean function.
-pub fn count10<F: FnMut(u64, u64, u64, u64, u64, u64, u64, u64, u64, u64) -> u64>(f: &mut F) -> u32 {
-    (f)(P0, P1, P2, P3, P4, P5, F, F, F, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, F, F, F, T).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, F, F, T, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, F, F, T, T).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, F, T, F, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, F, T, F, T).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, F, T, T, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, F, T, T, T).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, F, F, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, F, F, T).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, F, T, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, F, T, T).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, T, F, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, T, F, T).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, T, T, F).count_ones() +
-    (f)(P0, P1, P2, P3, P4, P5, T, T, T, T).count_ones()
+pub fn count10<F: FnMut(u64, u64, u64, u64, u64, u64, u64, u64, u64, u64) -> u64>(f: &mut F) -> u64 {
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, F, F, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, F, F, T)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, F, T, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, F, T, T)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, T, F, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, T, F, T)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, T, T, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, F, T, T, T)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, F, F, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, F, F, T)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, F, T, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, F, T, T)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, T, F, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, T, F, T)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, T, T, F)).count_ones() as u64 +
+    call(|| (f)(P0, P1, P2, P3, P4, P5, T, T, T, T)).count_ones() as u64
 }
 /// Counts the number of solutions of an n-argument boolean function.
 pub fn countn(n: usize, fun: &mut dyn FnMut(&[u64]) -> u64) -> u64 {
     match n {
-        0 => fun(&[]).count_ones() as u64,
+        0 => call(|| fun(&[])).count_ones() as u64,
         1 => count1(&mut |a| fun(&[a])) as u64,
         2 => count2(&mut |a, b| fun(&[a, b])) as u64,
         3 => count3(&mut |a, b, c| fun(&[a, b, c])) as u64,
@@ -621,7 +731,7 @@ pub fn countn(n: usize, fun: &mut dyn FnMut(&[u64]) -> u64) -> u64 {
                     args[8] = if (i & 0b100000000) == 0b100000000 {T} else {F};
                     sum += countn(n-9, &mut |vs: &[u64]| {
                         for i in 0..n-9 {args[i+9] = vs[i]}
-                        fun(&args)
+                        call(|| fun(&args))
                     });
                 }
                 sum
@@ -636,7 +746,7 @@ pub fn countn(n: usize, fun: &mut dyn FnMut(&[u64]) -> u64) -> u64 {
                     args[4] = if (i & 0b10000) == 0b10000 {T} else {F};
                     sum += countn(n-5, &mut |vs: &[u64]| {
                         for i in 0..n-5 {args[i+5] = vs[i]}
-                        fun(&args)
+                        call(|| fun(&args))
                     });
                 }
                 sum
@@ -650,140 +760,177 @@ pub fn countn(n: usize, fun: &mut dyn FnMut(&[u64]) -> u64) -> u64 {
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
 ///
 /// For 1-argument functions this is the same as normal counting of solutions.
-pub fn path1_count1<F: FnMut(u64) -> u64>(f: &mut F) -> u32 {count1(f)}
+pub fn path1_count1<F: FnMut(u64) -> u64>(f: &mut F) -> u64 {count1(f)}
 /// Path Semantical Logic: Counts the number of solutions of a 2-argument boolean function,
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
 ///
 /// For 2-argument functions this is the same as normal counting of solutions.
-pub fn path1_count2<F: FnMut(u64, u64) -> u64>(f: &mut F) -> u32 {count2(f)}
+pub fn path1_count2<F: FnMut(u64, u64) -> u64>(f: &mut F) -> u64 {count2(f)}
 /// Path Semantical Logic: Counts the number of solutions of a 3-argument boolean function,
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
 ///
 /// For 3-argument functions this is the same as normal counting of solutions.
-pub fn path1_count3<F: FnMut((u64, u64), u64) -> u64>(f: &mut F) -> u32 {
-    ((f)((0b00001111,
+pub fn path1_count3<F: FnMut((u64, u64), u64) -> u64>(f: &mut F) -> u64 {
+    call(|| ((f)((0b00001111,
           0b00110011),
           0b01010101,
-    ) & 0b11111111).count_ones()
+    ) & 0b11111111)).count_ones() as u64
 }
 /// Path Semantical Logic: Counts the number of solutions of a 4-argument boolean function,
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
-pub fn path1_count4<F: FnMut((u64, u64), (u64, u64)) -> u64>(f: &mut F) -> u32 {
-    ((f)((0b0000111111,
-          0b0011001111),
-         (0b0101010011,
-          0b0101010101)
-     ) & 0b1111111111).count_ones()
+pub fn path1_count4<F: FnMut((u64, u64), (u64, u64)) -> u64>(f: &mut F) -> u64 {
+    call(|| ((f)((0b00000011111111,
+          0b00111100001111),
+         (0b01001100110011,
+          0b01010101010101)
+     ) & 0b11111111111111)).count_ones() as u64
 }
 /// Path Semantical Logic: Counts the number of solutions of a 5-argument boolean function,
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
-pub fn path1_count5<F: FnMut((u64, u64, u64), (u64, u64)) -> u64>(f: &mut F) -> u32 {
-    ((f)((0b000000001111111111,
-          0b000011110000111111,
-          0b001100110011001111),
-         (0b010101010101010011,
-          0b010101010101010101))
-        & 0b11111_11111_11111_111).count_ones()
+pub fn path1_count5<F: FnMut((u64, u64, u64), (u64, u64)) -> u64>(f: &mut F) -> u64 {
+    call(|| ((f)((0b000000000011111111111111,
+          0b000011111100000011111111,
+          0b001100111100111100001111),
+         (0b010101001101001100110011,
+          0b010101010101010101010101))
+        & 0b11111_11111_11111_11111_1111)).count_ones() as u64
 }
 /// Path Semantical Logic: Counts the number of solutions of a 6-argument boolean function,
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
-pub fn path1_count6<F: FnMut((u64, u64, u64), (u64, u64, u64)) -> u64>(f: &mut F) -> u32 {
-    ((f)((0b0000000011111111111111,
-          0b0000111100001111111111,
-          0b0011001100110011111111),
-         (0b0101010101010100001111,
-          0b0101010101010100110011,
-          0b0101010101010101010101))
-         & 0b11111_11111_11111_11111_11).count_ones()
+pub fn path1_count6<F: FnMut((u64, u64, u64), (u64, u64, u64)) -> u64>(f: &mut F) -> u64 {
+    call(|| ((f)((0b0000000000000011111111111111111111111111,
+          0b0000111111111100000000001111111111111111,
+          0b0011001111111100111111110000000011111111),
+         (0b0101010000111101000011110000111100001111,
+          0b0101010011001101001100110011001100110011,
+          0b0101010101010101010101010101010101010101))
+         & 0b11111_11111_11111_11111_11111_11111_11111_11111)).count_ones() as u64
 }
 /// Path Semantical Logic: Counts the number of solutions of a 7-argument boolean function,
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
-pub fn path1_count7<F: FnMut((u64, u64, u64, u64), (u64, u64, u64)) -> u64>(f: &mut F) -> u32 {
-    ((f)((0b00000000000000001111111111111111111111,
-          0b00000000111111110000000011111111111111,
-          0b00001111000011110000111100001111111111,
-          0b00110011001100110011001100110011111111),
-         (0b01010101010101010101010101010100001111,
-          0b01010101010101010101010101010100110011,
-          0b01010101010101010101010101010101010101)
-    ) & 0b11111_11111_11111_11111_11111_11111_11111_111).count_ones()
+pub fn path1_count7<F: FnMut((u64, u64, u64, u64), (u64, u64, u64)) -> u64>(f: &mut F) -> u64 {
+    call(|| ((f)((0b00000000000000000000001111111111111111111111111111111111111111,
+          0b00000000111111111111110000000000000011111111111111111111111111,
+          0b00001111000011111111110000111111111100000000001111111111111111,
+          0b00110011001100111111110011001111111100111111110000000011111111),
+         (0b01010101010101000011110101010000111101000011110000111100001111,
+          0b01010101010101001100110101010011001101001100110011001100110011,
+          0b01010101010101010101010101010101010101010101010101010101010101)
+    ) & 0b11111_11111_11111_11111_11111_11111_11111_11111_11111_11111_11111_11111_11)).count_ones() as u64
 }
 /// Path Semantical Logic: Counts the number of solutions of a 8-argument boolean function,
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
-pub fn path1_count8<F: FnMut((u64, u64, u64, u64), (u64, u64, u64, u64)) -> u64>(f: &mut F) -> u32 {
-    ((f)((0b0000000000000000111111111111111111111111111111,
-          0b0000000011111111000000001111111111111111111111,
-          0b0000111100001111000011110000111111111111111111,
-          0b0011001100110011001100110011001111111111111111),
-         (0b0101010101010101010101010101010000000011111111,
-          0b0101010101010101010101010101010000111100001111,
-          0b0101010101010101010101010101010011001100110011,
-          0b0101010101010101010101010101010101010101010101))
-      & 0b11111_11111_11111_11111_11111_11111_11111_11111_11111_1).count_ones()
+pub fn path1_count8<F: FnMut((u64, u64, u64, u64), (u64, u64, u64, u64)) -> u64>(f: &mut F) -> u64 {
+    call(|| ((f)((0b0000000000000000000000000000001111111111111111111111111111111111,
+          0b0000000011111111111111111111110000000000000000000000111111111111,
+          0b0000111100001111111111111111110000111111111111111111000000000000,
+          0b0011001100110011111111111111110011001111111111111111001111111111),
+         (0b0101010101010100000000111111110101010000000011111111010000000011,
+          0b0101010101010100001111000011110101010000111100001111010000111100,
+          0b0101010101010100110011001100110101010011001100110011010011001100,
+          0b0101010101010101010101010101010101010101010101010101010101010101)))).count_ones() as u64 +
+    call(|| ((f)((0b11111111111111111111111111111111111111,
+          0b11111111111111111111111111111111111111,
+          0b00000011111111111111111111111111111111,
+          0b11111100000000000000001111111111111111),
+         (0b11111100000000111111110000000011111111,
+          0b00111100001111000011110000111100001111,
+          0b11001100110011001100110011001100110011,
+          0b01010101010101010101010101010101010101))
+       & 0b11111_11111_11111_11111_11111_11111_11111_111)).count_ones() as u64
 }
 /// Path Semantical Logic: Counts the number of solutions of a 9-argument boolean function,
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
-pub fn path1_count9<F: FnMut((u64, u64, u64, u64, u64), (u64, u64, u64, u64)) -> u64>(f: &mut F) -> u32 {
-    ((f)((0b0000000000000000000000000000000011111111111111111111111111111111,
-          0b0000000000000000111111111111111100000000000000001111111111111111,
-          0b0000000011111111000000001111111100000000111111110000000011111111,
-          0b0000111100001111000011110000111100001111000011110000111100001111,
-          0b0011001100110011001100110011001100110011001100110011001100110011),
-         (0b0101010101010101010101010101010101010101010101010101010101010100,
-          0b0101010101010101010101010101010101010101010101010101010101010100,
-          0b0101010101010101010101010101010101010101010101010101010101010100,
-          0b0101010101010101010101010101010101010101010101010101010101010101))).count_ones() +
-    ((f)((0b11111111111111,
-          0b11111111111111,
-          0b11111111111111,
-          0b11111111111111,
-          0b11111111111111),
-         (0b00000011111111,
-          0b00111100001111,
-          0b11001100110011,
-          0b01010101010101))
-        & 0b11111_11111_1111).count_ones()
+pub fn path1_count9<F: FnMut((u64, u64, u64, u64, u64), (u64, u64, u64, u64)) -> u64>(f: &mut F) -> u64 {
+    call(|| ((f)((0b0000000000000000000000000000000000000000000000111111111111111111,
+          0b0000000000000000111111111111111111111111111111000000000000000000,
+          0b0000000011111111000000001111111111111111111111000000001111111111,
+          0b0000111100001111000011110000111111111111111111000011110000111111,
+          0b0011001100110011001100110011001111111111111111001100110011001111),
+         (0b0101010101010101010101010101010000000011111111010101010101010000,
+          0b0101010101010101010101010101010000111100001111010101010101010000,
+          0b0101010101010101010101010101010011001100110011010101010101010011,
+          0b0101010101010101010101010101010101010101010101010101010101010101)))).count_ones() as u64 +
+    call(|| ((f)((0b1111111111111111111111111111111111111111111111111111111111111111,
+          0b0000000000001111111111111111111111111111111111111111111111111111,
+          0b1111111111110000000000000000000000111111111111111111111111111111,
+          0b1111111111110000111111111111111111000000000000000000111111111111,
+          0b1111111111110011001111111111111111001111111111111111000000000000),
+         (0b0000111111110101010000000011111111010000000011111111000000001111,
+          0b1111000011110101010000111100001111010000111100001111000011110000,
+          0b0011001100110101010011001100110011010011001100110011001100110011,
+          0b0101010101010101010101010101010101010101010101010101010101010101)))).count_ones() as u64 +
+    call(|| ((f)((0b11111111111111111111,
+          0b11111111111111111111,
+          0b11111111111111111111,
+          0b11111111111111111111,
+          0b00001111111111111111),
+         (0b11110000000011111111,
+          0b11110000111100001111,
+          0b00110011001100110011,
+          0b01010101010101010101))
+        & 0b11111_11111_11111_11111)).count_ones() as u64
 }
 /// Path Semantical Logic: Counts the number of solutions of a 10-argument boolean function,
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
-pub fn path1_count10<F: FnMut((u64, u64, u64, u64, u64), (u64, u64, u64, u64, u64)) -> u64>(f: &mut F) -> u32 {
-    ((f)((0b0000000000000000000000000000000011111111111111111111111111111111,
-          0b0000000000000000111111111111111100000000000000001111111111111111,
-          0b0000000011111111000000001111111100000000111111110000000011111111,
-          0b0000111100001111000011110000111100001111000011110000111100001111,
-          0b0011001100110011001100110011001100110011001100110011001100110011),
-         (0b0101010101010101010101010101010101010101010101010101010101010100,
-          0b0101010101010101010101010101010101010101010101010101010101010100,
-          0b0101010101010101010101010101010101010101010101010101010101010100,
-          0b0101010101010101010101010101010101010101010101010101010101010100,
-          0b0101010101010101010101010101010101010101010101010101010101010101))).count_ones() +
-    ((f)((0b111111111111111111111111111111,
-          0b111111111111111111111111111111,
-          0b111111111111111111111111111111,
-          0b111111111111111111111111111111,
-          0b111111111111111111111111111111),
-         (0b000000000000001111111111111111,
-          0b000000111111110000000011111111,
-          0b001111000011110000111100001111,
-          0b110011001100110011001100110011,
-          0b010101010101010101010101010101))
-        & 0b11111_11111_11111_11111_11111_11111).count_ones()
+pub fn path1_count10<F: FnMut((u64, u64, u64, u64, u64), (u64, u64, u64, u64, u64)) -> u64>(f: &mut F) -> u64 {
+    call(|| ((f)((0b0000000000000000000000000000000000000000000000000000000000000011,
+          0b0000000000000000111111111111111111111111111111111111111111111100,
+          0b0000000011111111000000001111111111111111111111111111111111111100,
+          0b0000111100001111000011110000111111111111111111111111111111111100,
+          0b0011001100110011001100110011001111111111111111111111111111111100),
+         (0b0101010101010101010101010101010000000000000000111111111111111101,
+          0b0101010101010101010101010101010000000011111111000000001111111101,
+          0b0101010101010101010101010101010000111100001111000011110000111101,
+          0b0101010101010101010101010101010011001100110011001100110011001101,
+          0b0101010101010101010101010101010101010101010101010101010101010101)))).count_ones() as u64 +
+    call(|| ((f)((0b1111111111111111111111111111111111111111111111111111111111111111,
+          0b0000000000000000000000000000000000000000000011111111111111111111,
+          0b0000001111111111111111111111111111111111111100000000000000000000,
+          0b0011110000111111111111111111111111111111111100001111111111111111,
+          0b1100110011001111111111111111111111111111111100110011111111111111),
+         (0b0101010101010000000000000000111111111111111101010100000000000000,
+          0b0101010101010000000011111111000000001111111101010100000000111111,
+          0b0101010101010000111100001111000011110000111101010100001111000011,
+          0b0101010101010011001100110011001100110011001101010100110011001100,
+          0b0101010101010101010101010101010101010101010101010101010101010101)))).count_ones() as u64 +
+    call(|| ((f)((0b1111111111111111111111111111111111111111111111111111111111111111,
+          0b1111111111111111111111111111111111111111111111111111111111111111,
+          0b0000000000000000001111111111111111111111111111111111111111111111,
+          0b1111111111111111110000000000000000000000000000000000111111111111,
+          0b1111111111111111110011111111111111111111111111111111000000000000),
+         (0b0011111111111111110100000000000000001111111111111111000000000000,
+          0b1100000000111111110100000000111111110000000011111111000000001111,
+          0b1100001111000011110100001111000011110000111100001111000011110000,
+          0b1100110011001100110100110011001100110011001100110011001100110011,
+          0b0101010101010101010101010101010101010101010101010101010101010101)))).count_ones() as u64 +
+    call(|| ((f)((0b1111111111111111111111111111111111111111111111111111,
+          0b1111111111111111111111111111111111111111111111111111,
+          0b1111111111111111111111111111111111111111111111111111,
+          0b1111111111111111111111111111111111111111111111111111,
+          0b0000000000000000000011111111111111111111111111111111),
+         (0b0000111111111111111100000000000000001111111111111111,
+          0b1111000000001111111100000000111111110000000011111111,
+          0b1111000011110000111100001111000011110000111100001111,
+          0b0011001100110011001100110011001100110011001100110011,
+          0b0101010101010101010101010101010101010101010101010101))
+        & 0b11111_11111_11111_11111_11111_11111_11111_11111_11111_11111_11)).count_ones() as u64
 }
 /// Path Semantical Logic: Counts the number of solutions of a n-argument boolean function,
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
 pub fn path1_countn(n: usize, fun: &mut dyn FnMut(&[u64], &[u64]) -> u64) -> u64 {
     match n {
-        0 => fun(&[], &[]).count_ones() as u64,
+        0 => call(|| fun(&[], &[])).count_ones() as u64,
         1 => path1_count1(&mut |a| fun(&[a], &[])) as u64,
         2 => path1_count2(&mut |a, b| fun(&[a], &[b])) as u64,
         3 => path1_count3(&mut |(a, b), c| fun(&[a, b], &[c])) as u64,
@@ -820,11 +967,21 @@ pub fn path1_countnm(f: usize, x: usize, fun: &mut dyn FnMut(&[u64], &[u64]) ->
     // Set `xs` to first and last case.
     for x in &mut xs {*x = 0b01};
     // Enumerate all cases when `fs` are ones.
-    let fs = vec![T; f];
+    let mut fs = vec![T; f];
     let mut all_fs_one = countn(x, &mut |xs| fun(&*fs, xs));
     // Subtract first and last case.
-    all_fs_one -= (fun(&*fs, &*xs) & 0b11).count_ones() as u64;
-    all_xs_zero + all_xs_one + all_fs_one
+    all_fs_one -= call(|| (fun(&*fs, &*xs) & 0b11)).count_ones() as u64;
+    let mut sum_fs_zero = 0;
+    // Set one zero in `fs` by turn.
+    for i in 0..f {
+        fs[i] = F;
+        // Enumerate all cases when there is one zero in `fs`.
+        sum_fs_zero += countn(x, &mut |xs| fun(&*fs, xs));
+        // Subtract first and last case.
+        sum_fs_zero -= call(|| (fun(&*fs, &*xs) & 0b11)).count_ones() as u64;
+        fs[i] = T;
+    }
+    all_xs_zero + all_xs_one + all_fs_one + sum_fs_zero
 }
 
 /// Returns `true` if proposition is correct, `false` otherwise.
@@ -878,43 +1035,43 @@ pub fn path1_prove3<F: FnMut((u64, u64), u64) -> u64>(f: &mut F) -> bool {path1_
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
 pub fn path1_prove4<F: FnMut((u64, u64), (u64, u64)) -> u64>(f: &mut F) -> bool {
-    path1_count4(f) == 10
+    path1_count4(f) == 14
 }
 /// Path Semantical Logic: Returns `true` if proposition is correct, `false` otherwise.
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
 pub fn path1_prove5<F: FnMut((u64, u64, u64), (u64, u64)) -> u64>(f: &mut F) -> bool {
-    path1_count5(f) == 18
+    path1_count5(f) == 24
 }
 /// Path Semantical Logic: Returns `true` if proposition is correct, `false` otherwise.
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
 pub fn path1_prove6<F: FnMut((u64, u64, u64), (u64, u64, u64)) -> u64>(f: &mut F) -> bool {
-    path1_count6(f) == 22
+    path1_count6(f) == 40
 }
 /// Path Semantical Logic: Returns `true` if proposition is correct, `false` otherwise.
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
 pub fn path1_prove7<F: FnMut((u64, u64, u64, u64), (u64, u64, u64)) -> u64>(f: &mut F) -> bool {
-    path1_count7(f) == 38
+    path1_count7(f) == 62
 }
 /// Path Semantical Logic: Returns `true` if proposition is correct, `false` otherwise.
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
 pub fn path1_prove8<F: FnMut((u64, u64, u64, u64), (u64, u64, u64, u64)) -> u64>(f: &mut F) -> bool {
-    path1_count8(f) == 46
+    path1_count8(f) == 102
 }
 /// Path Semantical Logic: Returns `true` if proposition is correct, `false` otherwise.
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
 pub fn path1_prove9<F: FnMut((u64, u64, u64, u64, u64), (u64, u64, u64, u64)) -> u64>(f: &mut F) -> bool {
-    path1_count9(f) == 78
+    path1_count9(f) == 148
 }
 /// Path Semantical Logic: Returns `true` if proposition is correct, `false` otherwise.
 ///
 /// For more information, see the section "Path Semantical Logic" at the top level documentation.
 pub fn path1_prove10<F: FnMut((u64, u64, u64, u64, u64), (u64, u64, u64, u64, u64)) -> u64>(f: &mut F) -> bool {
-    path1_count10(f) == 94
+    path1_count10(f) == 244
 }
 /// Path Semantical Logic: Returns `true` if proposition is correct, `false` otherwise.
 ///
@@ -932,7 +1089,7 @@ pub fn path1_proven<F: FnMut(&[u64], &[u64]) -> u64>(n: usize, fun: &mut F) -> b
 pub fn path1_lennm(f: usize, x: usize) -> u64 {
     let f = f as u32;
     let x = x as u32;
-    2_u64.pow(1 + f) + 2_u64.pow(x) - 2
+    2_u64.pow(1 + f) + (1 + f as u64) * (2_u64.pow(x) - 2)
 }
 
 /// Path Semantical Logic: Returns `true` if proposition is correct, `false` otherwise.
